/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.query.DefectQueryCheck;
import org.databene.jdbacl.SQLUtil;
import org.w3c.dom.Element;

/**
 * Parses the &lt;numberSize&gt; part of a &lt;check&gt; and creates a SQL defect query.<br/><br/>
 * Created: 26.05.2011 12:39:04
 * @since 0.8
 * @author Volker Bergmann
 */
public class NumberSizeParser extends DbSanityXMLParser {

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("expression");
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			"totalDigits", "fractionDigits", "min", "max");

	public NumberSizeParser() {
		super("numberSize", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, DefectQueryCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
	    DefectQueryCheck parent = (DefectQueryCheck) this.parent(parentPath);
		String table = parent.getTable();
		if (StringUtil.isEmpty(table))
			throw new SyntaxError("Missing 'table' spec in outer <check>", XMLUtil.format(element));

		// parse XML element
		String expression = getOptionalAttribute("expression", element);		
		String totalDigitsSpec = getOptionalAttribute("totalDigits", element);		
		String fractionDigitsSpec = getOptionalAttribute("fractionDigits", element);
		String minSpec = getOptionalAttribute("min", element);		
		String maxSpec = getOptionalAttribute("max", element);
		if (StringUtil.isEmpty(totalDigitsSpec) && StringUtil.isEmpty(fractionDigitsSpec) && 
				StringUtil.isEmpty(minSpec) && StringUtil.isEmpty(maxSpec))
			throw new SyntaxError("'min', 'max', 'totalDigits' or 'fractionDigits' must be specified", XMLUtil.format(element));

		Integer totalDigits = (totalDigitsSpec != null ? Integer.parseInt(totalDigitsSpec) : null);
		Integer fractionDigits = (fractionDigitsSpec != null ? Integer.parseInt(fractionDigitsSpec) : null);
		Integer min = (minSpec != null ? Integer.parseInt(minSpec) : null);
		Integer max = (maxSpec != null ? Integer.parseInt(maxSpec) : null);
		
		// create query
		StringBuilder builder = new StringBuilder();
		StringBuilder condition = builder;
		if (min != null)
			SQLUtil.addOptionalCondition(expression + " < " + min, condition);
		if (max != null)
			SQLUtil.addOptionalCondition(expression + " > " + max, condition);
		int postfixDigits = (fractionDigits != null ? fractionDigits : 0);
		if (totalDigits != null)
			SQLUtil.addOptionalCondition(expression + " > POWER(10, " + (totalDigits - postfixDigits) + ")", condition);
		if (fractionDigits != null)
			SQLUtil.addOptionalCondition(expression + " != round(" + expression + ", " + postfixDigits + ")", condition);
		String defectQuery = "select * from " + table + " where " + condition.toString();
		parent.setQuery(defectQuery);
		// TODO v0.8.2 imply default defectType if the parent does not have one
	    return defectQuery;
	}

}
